@@ -42,7 +42,44 @@ namespace Mono.WebServer.Apache
 {
 	public class Server : MarshalByRefObject
 	{
-		
+		sealed class ApplicationSettings
+		{
+			public string Apps;
+			public string AppConfigDir;
+			public string AppConfigFile;
+			public string RootDir;
+			public object Oport = 8080;
+			public string IP = "0.0.0.0";
+			public Exception Exception;
+			public bool NonStop;
+			public bool Verbose;
+			public bool Master;
+			public string FileName;
+			
+			public ApplicationSettings ()
+			{
+				this.Exception = null;
+				try {
+					this.Apps = AppSettings ["MonoApplications"];
+					this.AppConfigDir = AppSettings ["MonoApplicationsConfigDir"];
+					this.AppConfigFile = AppSettings ["MonoApplicationsConfigFile"];
+					this.RootDir = AppSettings ["MonoServerRootDir"];
+					this.IP = AppSettings ["MonoServerAddress"];
+					this.Oport = AppSettings ["MonoServerPort"];
+					this.FileName = AppSettings ["MonoUnixSocket"];
+
+					if (IP == null || IP.Length == 0)
+						IP = "0.0.0.0";
+					if (Oport == null)
+						Oport = 8080;
+				} catch (Exception ex) {
+					Console.Error.WriteLine ("Exception caught during reading the configuration file:");
+					Console.Error.WriteLine (ex);
+					this.Exception = ex;
+				}
+			}
+		}
+
 		static void ShowVersion ()
 		{
 			Assembly assembly = Assembly.GetExecutingAssembly ();
@@ -76,6 +113,7 @@ namespace Mono.WebServer.Apache
 			Console.WriteLine ("                    Default value: 127.0.0.1");
 			Console.WriteLine ("                    AppSettings key name: MonoServerAddress");
 			Console.WriteLine ();
+			Console.WriteLine ("    --backlog N:    the listen backlog. Default value: 500");
 			Console.WriteLine ("    --root rootdir: the server changes to this directory before");
 			Console.WriteLine ("                    anything else.");
 			Console.WriteLine ("                    Default value: current directory.");
@@ -114,6 +152,10 @@ namespace Mono.WebServer.Apache
 			Console.WriteLine ("                    Default value: /:.");
 			Console.WriteLine ("                    AppSettings key name: MonoApplications");
 			Console.WriteLine ();
+			Console.WriteLine ("    --minThreads N:    the minimum number of threads the thread pool creates on startup.");
+			Console.WriteLine ("                       Increase this value to handle a sudden inflow of connections.");
+			Console.WriteLine ("                       Default value: (runtime default)");
+			Console.WriteLine ("    --backlog N:    the listen backlog. Default value: 500");
 			Console.WriteLine ("    --terminate: gracefully terminates a running mod-mono-server instance.");
 			Console.WriteLine ("                 All other options but --filename or --address and --port");
 			Console.WriteLine ("                 are ignored if this option is provided.");
@@ -152,8 +194,8 @@ namespace Mono.WebServer.Apache
 		{
 			if ((options & value) != 0) {
 				ShowHelp ();
-				Console.WriteLine ();
-				Console.WriteLine ("ERROR: Option '{0}' duplicated.", name);
+				Console.Error.WriteLine ();
+				Console.Error.WriteLine ("ERROR: Option '{0}' duplicated.", name);
 				Environment.Exit (1);
 			}
 
@@ -161,31 +203,25 @@ namespace Mono.WebServer.Apache
 			if ((options & Options.FileName) != 0 &&
 			    ((options & Options.Port) != 0 || (options & Options.Address) != 0)) {
 				ShowHelp ();
-				Console.WriteLine ();
-				Console.WriteLine ("ERROR: --port/--address and --filename are mutually exclusive");
+				Console.Error.WriteLine ();
+				Console.Error.WriteLine ("ERROR: --port/--address and --filename are mutually exclusive");
 				Environment.Exit (1);
 			}
 		}
 
 		static NameValueCollection AppSettings {
-			get {
-#if NET_2_0
-				return ConfigurationManager.AppSettings;
-#else
-				return ConfigurationSettings.AppSettings;
-#endif
-			}
+			get { return ConfigurationManager.AppSettings; }
 		}
 
 		public static void CurrentDomain_UnhandledException (object sender, UnhandledExceptionEventArgs e)
 		{
 			Exception ex = (Exception)e.ExceptionObject;
 
-			Console.WriteLine ("Handling exception type {0}", ex.GetType ().Name);
-			Console.WriteLine ("Message is {0}", ex.Message);
-			Console.WriteLine ("IsTerminating is set to {0}", e.IsTerminating);
+			Console.Error.WriteLine ("Handling exception type {0}", ex.GetType ().Name);
+			Console.Error.WriteLine ("Message is {0}", ex.Message);
+			Console.Error.WriteLine ("IsTerminating is set to {0}", e.IsTerminating);
 			if (e.IsTerminating)
-				Console.WriteLine (ex);
+				Console.Error.WriteLine (ex);
 		}
 
 		
@@ -193,37 +229,36 @@ namespace Mono.WebServer.Apache
 		{
 			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler (CurrentDomain_UnhandledException);
 			bool quiet = false;
-			while (true) {
+			//while (true) {
 				try {
-					return new Server ().RealMain (args, true, null, quiet);
+					Server svr = new Server ();
+					return svr.RealMain (args, true, null, quiet);
 				} catch (ThreadAbortException) {
 					// Single-app mode and ASP.NET appdomain unloaded
 					Thread.ResetAbort ();
 					quiet = true; // hush 'RealMain'
 				}
-			}
+			//}
+			return 1;
 		}
 
+		//
+		// Parameters:
+		//
+		//   args - original args passed to the program
+		//   root - true means caller is in the root domain
+		//   ext_apphost - used when single app mode is used, in a recursive call to
+		//        RealMain from the single app domain
+		//   quiet - don't show messages. Used to avoid double printing of the banner
+		//
 		public int RealMain (string [] args, bool root, IApplicationHost ext_apphost, bool quiet)
 		{
-			bool nonstop = false;
-			bool verbose = false;
-			string apps = AppSettings ["MonoApplications"];
-			string appConfigDir = AppSettings ["MonoApplicationsConfigDir"];
-			string appConfigFile = AppSettings ["MonoApplicationsConfigFile"];
-			string rootDir = AppSettings ["MonoServerRootDir"];
-			object oport;
-			string ip = AppSettings ["MonoServerAddress"];
-			bool master = false;
-			string filename = AppSettings ["MonoUnixSocket"];
-			if (ip == "" || ip == null)
-				ip = "0.0.0.0";
-
-			oport = AppSettings ["MonoServerPort"];
-			if (oport == null)
-				oport = 8080;
+			ApplicationSettings settings = new ApplicationSettings ();
+			if (ext_apphost != null)
+				settings.RootDir = ext_apphost.Path;
 
 			Options options = 0;
+			int backlog = 500;
 			int hash = 0;
 			for (int i = 0; i < args.Length; i++){
 				string a = args [i];
@@ -233,41 +268,64 @@ namespace Mono.WebServer.Apache
 				switch (a){
 				case "--filename":
 					CheckAndSetOptions (a, Options.FileName, ref options);
-					filename = args [++i];
+					settings.FileName = args [++i];
 					break;
 				case "--terminate":
 					CheckAndSetOptions (a, Options.Terminate, ref options);
 					break;
 				case "--master":
 					CheckAndSetOptions (a, Options.Master, ref options);
-					master = true;
+					settings.Master = true;
 					break;
 				case "--port":
 					CheckAndSetOptions (a, Options.Port, ref options);
-					oport = args [++i];
+					settings.Oport = args [++i];
 					break;
 				case "--address":
 					CheckAndSetOptions (a, Options.Address, ref options);
-					ip = args [++i];
+					settings.IP = args [++i];
+					break;
+				case "--backlog":
+					string backlogstr = args [++i];
+					try {
+						backlog = Convert.ToInt32 (backlogstr);
+					} catch (Exception) {
+						Console.WriteLine ("The value given for backlog is not valid {0}", backlogstr);
+						return 1;
+					}
 					break;
 				case "--root":
 					CheckAndSetOptions (a, Options.Root, ref options);
-					rootDir = args [++i];
+					settings.RootDir = args [++i];
 					break;
 				case "--applications":
 					CheckAndSetOptions (a, Options.Applications, ref options);
-					apps = args [++i];
+					settings.Apps = args [++i];
 					break;
 				case "--appconfigfile":
 					CheckAndSetOptions (a, Options.AppConfigFile, ref options);
-					appConfigFile = args [++i];
+					settings.AppConfigFile = args [++i];
 					break;
 				case "--appconfigdir":
 					CheckAndSetOptions (a, Options.AppConfigDir, ref options);
-					appConfigDir = args [++i];
+					settings.AppConfigDir = args [++i];
+					break;
+				case "--minThreads":
+					string mtstr = args [++i];
+					int minThreads = 0;
+					try {
+						minThreads = Convert.ToInt32 (mtstr);
+					} catch (Exception) {
+						Console.WriteLine ("The value given for minThreads is not valid {0}", mtstr);
+						return 1;
+					}
+
+					if (minThreads > 0)
+						ThreadPool.SetMinThreads(minThreads, minThreads);
+
 					break;
 				case "--nonstop":
-					nonstop = true;
+					settings.NonStop = true;
 					break;
 				case "--help":
 					ShowHelp ();
@@ -276,7 +334,7 @@ namespace Mono.WebServer.Apache
 					ShowVersion ();
 					return 0;
 				case "--verbose":
-					verbose = true;
+					settings.Verbose = true;
 					break;
 				case "--pidfile": {
 					string pidfile = args[++i];
@@ -294,7 +352,7 @@ namespace Mono.WebServer.Apache
 					MonoWorkerRequest.CheckFileAccess = false;
 					break;
 				default:
-					Console.WriteLine ("Unknown argument: {0}", a);
+					Console.Error.WriteLine ("Unknown argument: {0}", a);
 					ShowHelp ();
 					return 1;
 				}
@@ -306,15 +364,15 @@ namespace Mono.WebServer.Apache
 			string lockfile;
 			bool useTCP = ((options & Options.Port) != 0);
 			if (!useTCP) {
-				if (filename == null || filename == "")
-					filename = "/tmp/mod_mono_server";
+				if (settings.FileName == null || settings.FileName.Length == 0)
+					settings.FileName = "/tmp/mod_mono_server";
 
 				if ((options & Options.Address) != 0) {
 					ShowHelp ();
-					Console.WriteLine ();
-					Console.WriteLine ("ERROR: --address without --port");
+					Console.Error.WriteLine ();
+					Console.Error.WriteLine ("ERROR: --address without --port");
 					Environment.Exit (1);
-				} lockfile = Path.Combine (Path.GetTempPath (), Path.GetFileName (filename));
+				} lockfile = Path.Combine (Path.GetTempPath (), Path.GetFileName (settings.FileName));
 				lockfile = String.Format ("{0}_{1}", lockfile, hash);
 			} else {
 				lockfile = Path.Combine (Path.GetTempPath (), "mod_mono_TCP_");
@@ -324,63 +382,63 @@ namespace Mono.WebServer.Apache
 			IPAddress ipaddr = null;
 			ushort port;
 			try {
-				port = Convert.ToUInt16 (oport);
+				port = Convert.ToUInt16 (settings.Oport);
 			} catch (Exception) {
-				Console.WriteLine ("The value given for the listen port is not valid: " + oport);
+				Console.Error.WriteLine ("The value given for the listen port is not valid: " + settings.Oport);
 				return 1;
 			}
 
 			try {
-				ipaddr = IPAddress.Parse (ip);
+				ipaddr = IPAddress.Parse (settings.IP);
 			} catch (Exception) {
-				Console.WriteLine ("The value given for the address is not valid: " + ip);
+				Console.Error.WriteLine ("The value given for the address is not valid: " + settings.IP);
 				return 1;
 			}
 
-			if (rootDir != null && rootDir != "") {
+			if (settings.RootDir != null && settings.RootDir.Length > 0) {
 				try {
-					Environment.CurrentDirectory = rootDir;
+					Environment.CurrentDirectory = settings.RootDir;
 				} catch (Exception e) {
-					Console.WriteLine ("Error: {0}", e.Message);
+					Console.Error.WriteLine ("Error: {0}", e.Message);
 					return 1;
 				}
 			}
 
-			rootDir = Directory.GetCurrentDirectory ();
+			settings.RootDir = Directory.GetCurrentDirectory ();
 			
 			WebSource webSource;
 			if (useTCP) {
 				webSource = new ModMonoTCPWebSource (ipaddr, port, lockfile);
 			} else {
-				webSource = new ModMonoWebSource (filename, lockfile);
+				webSource = new ModMonoWebSource (settings.FileName, lockfile);
 			}
 
 			if ((options & Options.Terminate) != 0) {
-				if (verbose)
-					Console.WriteLine ("Shutting down running mod-mono-server...");
+				if (settings.Verbose)
+					Console.Error.WriteLine ("Shutting down running mod-mono-server...");
 				
 				bool res = ((ModMonoWebSource) webSource).GracefulShutdown ();
-				if (verbose)
-					Console.WriteLine (res ? "Done." : "Failed");
+				if (settings.Verbose)
+					Console.Error.WriteLine (res ? "Done." : "Failed");
 
 				return (res) ? 0 : 1;
 			}
 
-			ApplicationServer server = new ApplicationServer (webSource);
-			server.Verbose = verbose;
+			ApplicationServer server = new ApplicationServer (webSource, settings.RootDir);
+			server.Verbose = settings.Verbose;
 			server.SingleApplication = !root;
 
 			Console.WriteLine (Assembly.GetExecutingAssembly ().GetName ().Name);
-			if (apps != null)
-				server.AddApplicationsFromCommandLine (apps);
+			if (settings.Apps != null)
+				server.AddApplicationsFromCommandLine (settings.Apps);
 
-			if (appConfigFile != null)
-				server.AddApplicationsFromConfigFile (appConfigFile);
+			if (settings.AppConfigFile != null)
+				server.AddApplicationsFromConfigFile (settings.AppConfigFile);
 
-			if (appConfigDir != null)
-				server.AddApplicationsFromConfigDirectory (appConfigDir);
+			if (settings.AppConfigDir != null)
+				server.AddApplicationsFromConfigDirectory (settings.AppConfigDir);
 
-			if (!master && apps == null && appConfigDir == null && appConfigFile == null)
+			if (!settings.Master && settings.Apps == null && settings.AppConfigDir == null && settings.AppConfigFile == null)
 				server.AddApplicationsFromCommandLine ("/:.");
 
 			VPathToHost vh = server.GetSingleApp ();
@@ -391,25 +449,26 @@ namespace Mono.WebServer.Apache
 				webSource.Dispose ();
 				return svr.RealMain (args, false, vh.AppHost, quiet);
 			}
-			if (ext_apphost != null)
+			if (ext_apphost != null) {
+				ext_apphost.Server = server;
 				server.AppHost = ext_apphost;
-
+			}
 			if (!useTCP && !quiet) {
-				Console.WriteLine ("Listening on: {0}", filename);
+				Console.Error.WriteLine ("Listening on: {0}", settings.FileName);
 			} else if (!quiet) {
-				Console.WriteLine ("Listening on port: {0}", port);
-				Console.WriteLine ("Listening on address: {0}", ip);
+				Console.Error.WriteLine ("Listening on port: {0}", port);
+				Console.Error.WriteLine ("Listening on address: {0}", settings.IP);
 			}
 
 			if (!quiet)
-				Console.WriteLine ("Root directory: {0}", rootDir);
+				Console.Error.WriteLine ("Root directory: {0}", settings.RootDir);
 
 			try {
-				if (server.Start (!nonstop) == false)
+				if (server.Start (!settings.NonStop, backlog) == false)
 					return 2;
 
-				if (!nonstop) {
-					Console.WriteLine ("Hit Return to stop the server.");
+				if (!settings.NonStop) {
+					Console.Error.WriteLine ("Hit Return to stop the server.");
 					while (true) {
 						try {
 							Console.ReadLine ();
@@ -424,12 +483,19 @@ namespace Mono.WebServer.Apache
 				}
 			} catch (Exception e) {
 				if (!(e is ThreadAbortException))
-					Console.WriteLine ("Error: {0}", e.Message);
+					Console.Error.WriteLine ("Error: {0}", e.Message);
+				else
+					server.ShutdownSockets ();
 				return 1;
 			}
 
 			return 0;
 		}
+
+		public override object InitializeLifetimeService ()
+		{
+			return null;
+		}
 	}
 }
 
