@@ -88,7 +88,7 @@ namespace Mono.WebServer.XSP
 			string copyright = ((AssemblyCopyrightAttribute) att [0]).Copyright;
 			att = assembly.GetCustomAttributes (typeof (AssemblyDescriptionAttribute), false);
 			string description = ((AssemblyDescriptionAttribute) att [0]).Description;
-			Console.WriteLine ("{0} {1}\n(c) {2}\n{3}",
+			Console.WriteLine ("{0} {1}\nCopyright {2}\n{3}",
 					Path.GetFileName (assembly.Location), version, copyright, description);
 		}
 		
@@ -109,6 +109,10 @@ namespace Mono.WebServer.XSP
 			Console.WriteLine ("                    Default value: 0.0.0.0");
 			Console.WriteLine ("                    AppSettings key name: MonoServerAddress");
 			Console.WriteLine ();
+			Console.WriteLine ("    --minThreads N:    the minimum number of threads the thread pool creates on startup.");
+			Console.WriteLine ("                       Increase this value to handle a sudden inflow of connections.");
+			Console.WriteLine ("                       Default value: (runtime default)");
+			Console.WriteLine ("    --backlog N:    the listen backlog. Default value: 500");
 			Console.WriteLine ("    --https:        enable SSL for the server");
 			Console.WriteLine ("                    Default value: false.");
 			Console.WriteLine ("                    AppSettings key name: ");
@@ -181,6 +185,8 @@ namespace Mono.WebServer.XSP
 			Console.WriteLine ();
 			Console.WriteLine ("    --no-hidden: allow access to hidden files (see 'man xsp' for details)");
 			Console.WriteLine ();
+			Console.WriteLine ("    --quiet: disables the initial start up information");
+			Console.WriteLine ();
 			Console.WriteLine ("    --version: displays version information and exits.");
 			Console.WriteLine ("    --verbose: prints extra messages. Mainly useful for debugging.");
 			Console.WriteLine ("    --pidfile file: write the process PID to the specified file.");
@@ -235,13 +241,7 @@ namespace Mono.WebServer.XSP
 		}
 
 		static NameValueCollection AppSettings {
-			get {
-#if NET_2_0
-				return ConfigurationManager.AppSettings;
-#else
-				return ConfigurationSettings.AppSettings;
-#endif
-			}
+			get { return ConfigurationManager.AppSettings; }
 		}
 
 		public static void CurrentDomain_UnhandledException (object sender, UnhandledExceptionEventArgs e)
@@ -263,6 +263,7 @@ namespace Mono.WebServer.XSP
 				try {
 					return new Server ().RealMain (args, true, null, quiet);
 				} catch (ThreadAbortException ex) {
+					Console.WriteLine (ex);
 					// Single-app mode and ASP.NET appdomain unloaded
 					Thread.ResetAbort ();
 					quiet = true; // hush 'RealMain'
@@ -270,6 +271,15 @@ namespace Mono.WebServer.XSP
 			}
 		}
 
+		//
+		// Parameters:
+		//
+		//   args - original args passed to the program
+		//   root - true means caller is in the root domain
+		//   ext_apphost - used when single app mode is used, in a recursive call to
+		//        RealMain from the single app domain
+		//   quiet - don't show messages. Used to avoid double printing of the banner
+		//
 		public int RealMain (string [] args, bool root, IApplicationHost ext_apphost, bool quiet)
 		{
 			SecurityConfiguration security = new SecurityConfiguration ();
@@ -283,6 +293,7 @@ namespace Mono.WebServer.XSP
 
 			Options options = 0;
 			int hash = 0;
+			int backlog = 500;
 			for (int i = 0; i < args.Length; i++){
 				string a = args [i];
 				int idx = (i + 1 < args.Length) ? i + 1 : i;
@@ -332,6 +343,15 @@ namespace Mono.WebServer.XSP
 					CheckAndSetOptions (a, Options.Address, ref options);
 					settings.IP = args [++i];
 					break;
+				case "--backlog":
+					string backlogstr = args [++i];
+					try {
+						backlog = Convert.ToInt32 (backlogstr);
+					} catch (Exception) {
+						Console.WriteLine ("The value given for backlog is not valid {0}", backlogstr);
+						return 1;
+					}
+					break;
 				case "--root":
 					CheckAndSetOptions (a, Options.Root, ref options);
 					settings.RootDir = args [++i];
@@ -347,6 +367,20 @@ namespace Mono.WebServer.XSP
 				case "--appconfigdir":
 					CheckAndSetOptions (a, Options.AppConfigDir, ref options);
 					settings.AppConfigDir = args [++i];
+					break;
+				case "--minThreads":
+					string mtstr = args [++i];
+					int minThreads = 0;
+					try {
+						minThreads = Convert.ToInt32 (mtstr);
+					} catch (Exception) {
+						Console.WriteLine ("The value given for minThreads is not valid {0}", mtstr);
+						return 1;
+					}
+
+					if (minThreads > 0)
+						ThreadPool.SetMinThreads(minThreads, minThreads);
+
 					break;
 				case "--nonstop":
 					settings.NonStop = true;
@@ -354,6 +388,9 @@ namespace Mono.WebServer.XSP
 				case "--help":
 					ShowHelp ();
 					return 0;
+				case "--quiet":
+					quiet = true;
+					break;
 				case "--version":
 					ShowVersion ();
 					return 0;
@@ -426,7 +463,7 @@ namespace Mono.WebServer.XSP
 				webSource = new XSPWebSource (ipaddr, port, !root);
 			}
 
-			ApplicationServer server = new ApplicationServer (webSource);
+			ApplicationServer server = new ApplicationServer (webSource, settings.RootDir);
 			server.Verbose = settings.Verbose;
 			server.SingleApplication = !root;
 
@@ -460,11 +497,13 @@ namespace Mono.WebServer.XSP
 			}
 
 			try {
-				if (server.Start (!settings.NonStop, settings.Exception) == false)
+				if (server.Start (!settings.NonStop, settings.Exception, backlog) == false)
 					return 2;
 				
-				if (!quiet)
+				if (!quiet) {
+					// MonoDevelop depends on this string. If you change it, let them know.
 					Console.WriteLine ("Listening on port: {0} {1}", server.Port, security);
+				}
 				if (port == 0 && !quiet)
 					Console.Error.WriteLine ("Random port: {0}", server.Port);
 				
@@ -494,6 +533,8 @@ namespace Mono.WebServer.XSP
 			} catch (Exception e) {
 				if (!(e is ThreadAbortException))
 					Console.WriteLine ("Error: {0}", e);
+				else
+					server.ShutdownSockets ();
 				return 1;
 			}
 
