@@ -35,6 +35,7 @@ using System.Xml;
 using System.Web;
 using System.Web.Hosting;
 using System.Collections;
+using System.Collections.Generic;
 using System.Text;
 using System.Threading;
 using System.IO;
@@ -90,6 +91,7 @@ namespace Mono.WebServer
 		Socket listen_socket;
 		bool single_app;
 		Exception initialException;
+		string physicalRoot;
 		
 		Thread runner;
 
@@ -113,11 +115,7 @@ namespace Mono.WebServer
 
 		public int Port {
 			get {
-				if (listen_socket == null
-#if NET_2_0
-				    || !listen_socket.IsBound
-#endif
-				)
+				if (listen_socket == null || !listen_socket.IsBound)
 					return -1;
 
 				IPEndPoint iep = listen_socket.LocalEndPoint as IPEndPoint;
@@ -127,10 +125,25 @@ namespace Mono.WebServer
 				return iep.Port;
 			}
 		}
-		
-		public ApplicationServer (WebSource source)
+
+		public string PhysicalRoot {
+			get { return physicalRoot; }
+		}
+
+		[Obsolete ("Use the .ctor that takes a 'physicalRoot' argument instead")]
+		public ApplicationServer (WebSource source) : this (source, Environment.CurrentDirectory)
 		{
-			webSource = source;
+		}
+
+		public ApplicationServer (WebSource source, string physicalRoot)
+		{
+			if (source == null)
+				throw new ArgumentNullException ("source");
+			if (physicalRoot == null || physicalRoot.Length == 0)
+				throw new ArgumentNullException ("physicalRoot");
+			
+			this.webSource = source;
+			this.physicalRoot = physicalRoot;
 		} 
 
 		public bool Verbose {
@@ -282,13 +295,13 @@ namespace Mono.WebServer
  			}
  		}
 
-		public bool Start (bool bgThread, Exception initialException)
+		public bool Start (bool bgThread, Exception initialException, int backlog)
 		{
 			this.initialException = initialException;
-			return Start (bgThread);
+			return Start (bgThread, backlog);
 		}
 		
-		public bool Start (bool bgThread)
+		public bool Start (bool bgThread, int backlog)
 		{
 			if (started)
 				throw new InvalidOperationException ("The server is already started.");
@@ -304,9 +317,9 @@ namespace Mono.WebServer
 				v.RequestBroker = webSource.CreateRequestBroker ();
 				AppHost.RequestBroker = v.RequestBroker;
 			}
+
 			listen_socket = webSource.CreateSocket ();
-			listen_socket.Listen (500);
-			listen_socket.Blocking = false;
+			listen_socket.Listen (backlog);
 			runner = new Thread (new ThreadStart (RunServer));
 			runner.IsBackground = bgThread;
 			runner.Start ();
@@ -330,13 +343,24 @@ namespace Mono.WebServer
 			stopThread.Start ();
 		}
 
+		public void ShutdownSockets ()
+		{
+			if (listen_socket != null) {
+				try {
+					listen_socket.Close ();
+				} catch {
+				} finally {
+					listen_socket = null;
+				}
+			}
+		}
+
 		void RealStop ()
 		{
 			started = false;
 			runner.Abort ();
-			listen_socket.Close ();
+			ShutdownSockets ();
 			UnloadAll ();
-			Thread.Sleep (1000);
 		}
 
 		public void UnloadAll ()
@@ -350,23 +374,24 @@ namespace Mono.WebServer
 
 		void SetSocketOptions (Socket sock)
 		{
-#if !MOD_MONO_SERVER
+
 			try {
+				sock.LingerState = new LingerOption (true, 15);
+#if !MOD_MONO_SERVER
 				sock.SetSocketOption (SocketOptionLevel.Socket, SocketOptionName.SendTimeout, 15000); // 15s
 				sock.SetSocketOption (SocketOptionLevel.Socket, SocketOptionName.ReceiveTimeout, 15000); // 15s
+#endif
 			} catch {
 				// Ignore exceptions here for systems that do not support these options.
 			}
-#endif
 		}
 
-
-		AsyncCallback accept_cb;
 		void RunServer ()
 		{
 			started = true;
-			accept_cb = new AsyncCallback (OnAccept);
-			listen_socket.BeginAccept (accept_cb, null);
+			SocketAsyncEventArgs args = new SocketAsyncEventArgs ();
+			args.Completed += OnAccept;
+			listen_socket.AcceptAsync (args);
 			if (runner.IsBackground)
 				return;
 
@@ -374,33 +399,60 @@ namespace Mono.WebServer
 				Thread.Sleep (1000000);
 		}
 
-		void OnAccept (IAsyncResult ares)
+		void OnAccept (object sender, EventArgs e)
 		{
-			Socket accepted = null;
+			if (!started && single_app) {
+				// We are shutting down. Last call...
+				Environment.Exit (0);
+			}
+
+			SocketAsyncEventArgs args = (SocketAsyncEventArgs) e;
+			Socket accepted = args.AcceptSocket;
+			args.AcceptSocket = null;
+
+			if (args.SocketError != SocketError.Success) {
+				CloseSocket(accepted);
+				accepted = null;
+			}
+
 			try {
-				accepted = listen_socket.EndAccept (ares);
-			} catch {
-			} finally {
-				if (started) {
-					try {
-						listen_socket.BeginAccept (accept_cb, null);
-					} catch (Exception ex) {
-						if (accepted != null) {
-							SendException (accepted, ex);
-							accepted.Close ();
-							throw;
-						}
-					}
-				}
+				if (started)
+					listen_socket.AcceptAsync (args);
+			} catch (Exception ex) {
+				if (accepted != null)
+					CloseSocket (accepted);
+
+				// not much we can do. fast fail by killing the process.
+				Console.Error.WriteLine ("Unable to accept socket. Exiting the process. {0}", ex);
+				Environment.Exit (1);
 			}
 
 			if (accepted == null)
 				return;
-			accepted.Blocking = true;
+
 			SetSocketOptions (accepted);
 			StartRequest (accepted, 0);
 		}
 
+		void CloseSocket(Socket socket)
+		{
+			if (socket == null)
+				return;
+
+			// attempt a quick RST of the connection
+			try {
+				socket.LingerState = new LingerOption (true, 0);
+			} catch {
+				// ignore
+			}
+
+			try {
+				socket.Close();
+			} catch {
+				// ignore
+			}
+		}
+
 		void SendException (Socket socket, Exception ex)
 		{
 			StringBuilder sb = new StringBuilder ();
@@ -451,7 +503,20 @@ namespace Mono.WebServer
 				else
 					worker.Run (null);
 			} catch (Exception) {
-				try { accepted.Close (); } catch {}
+				try {
+					if (accepted != null) {
+						try {
+							if (accepted.Connected)
+								accepted.Shutdown (SocketShutdown.Both);
+						} catch {
+							// ignore
+						}
+						
+						accepted.Close ();
+					}
+				} catch {
+					// ignore
+				}
 			}
 		}
 
@@ -469,8 +534,6 @@ namespace Mono.WebServer
 			VPathToHost bestMatch = null;
 			int bestMatchLength = 0;
 
-			// Console.WriteLine("GetApplicationForPath({0},{1},{2},{3})", vhost, port,
-			//			path, defaultToRoot);
 			for (int i = vpathToHost.Count - 1; i >= 0; i--) {
 				VPathToHost v = (VPathToHost) vpathToHost [i];
 				int matchLength = v.vpath.Length;
