@@ -34,55 +34,32 @@ using System.Text;
 using System.Text.RegularExpressions;
 using System.Web;
 using System.Web.Configuration;
-
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 
 namespace Mono.WebServer
 {
 	public class BaseApplicationHost : MarshalByRefObject, IApplicationHost
 	{
-#if NET_2_0
 		static readonly ReaderWriterLockSlim handlersCacheLock = new ReaderWriterLockSlim ();
-#else
-		static readonly ReaderWriterLock handlersCacheLock = new ReaderWriterLock ();
-#endif
 		
 		string path;
 		string vpath;
 		IRequestBroker requestBroker;
 		EndOfRequestHandler endOfRequest;
 		ApplicationServer appserver;
-#if NET_2_0
-		Dictionary <string, bool> handlersCache;
-#else
-		Hashtable handlersCache;
-#endif
-		
-		/// <summary>
-		///   Creates the <see cref="EndOfRequest"/> event handler and registers
-		///   a handler (<see cref="OnUnload"/>) with the <see cref="AppDomain.DomainUnload"/> event.
-		/// </summary>
+		Dictionary <string, bool> handlersCache;		
+
 		public BaseApplicationHost ()
 		{
 			endOfRequest = new EndOfRequestHandler (EndOfRequest);
 			AppDomain.CurrentDomain.DomainUnload += new EventHandler (OnUnload);
 		}
 
-		/// <summary>
-		///   Unloads the current application domain by calling <see cref="HttpRuntime.UnloadAppDomain"/>
-		/// </summary>
 		public void Unload ()
 		{
 			HttpRuntime.UnloadAppDomain ();
 		}
 
-		/// <summary>
-		///   Event handler for the <see cref="System.AppDomain.DomainUnload"/>
-		///   event. Calls <see cref="ApplicationServer.DestroyHost"/>
-		///   to shut the host down.
-		/// </summary>
 		public void OnUnload (object o, EventArgs args)
 		{
 			if (appserver != null)
@@ -94,17 +71,11 @@ namespace Mono.WebServer
 			return null; // who wants to live forever?
 		}
 
-		/// <summary>
-		///   Reference to the associated <see cref="ApplicationServer"/>
-		/// </summary>
 		public ApplicationServer Server {
 			get { return appserver; }
 			set { appserver = value; }
 		}
 
-		/// <summary>
-		///   Physical path to the application root directory.
-		/// </summary>
 		public string Path {
 			get {
 				if (path == null)
@@ -114,9 +85,6 @@ namespace Mono.WebServer
 			}
 		}
 
-		/// <summary>
-		///   Virtual path to the application root.
-		/// </summary>
 		public string VPath {
 			get {
 				if (vpath == null)
@@ -126,34 +94,16 @@ namespace Mono.WebServer
 			}
 		}
 
-		/// <summary>
-		///   Returns the current application domain.
-		/// </summary>
 		public AppDomain Domain {
 			get { return AppDomain.CurrentDomain; }
 		}
 
-		/// <summary>
-		///   Reference to the associated request broker
-		/// </summary>
 		public IRequestBroker RequestBroker
 		{
 			get { return requestBroker; }
 			set { requestBroker = value; }
 		}
 
-		/// <summary>
-		///   Process a request.
-		/// </summary>
-		/// <param name="mwr">A worker object to actually process the request</param>
-		/// <remarks>
-		///   If the mwr parameter is null or no request data can be read, the request will be ended
-		///   immediately. Otherwise, registers an event handler for the worker's <see
-		///   cref="MonoWorkerRequest.EndOfRequest"/> event and calls the worker's <see
-		///   cref="MonoWorkerRequest.ProcessRequest"/> method to actually process the request. If an unhandled exception
-		///   occurs during that phase, it is printed to the console and <see cref="EndOfRequest"/> is called
-		///   immediately.
-		/// </remarks>
 		protected void ProcessRequest (MonoWorkerRequest mwr)
 		{
 			if (mwr == null) {
@@ -192,9 +142,6 @@ namespace Mono.WebServer
 			}
 		}
 
-		/// <summary>
-		///    Checks if the passed URI maps to a HTTP handler
-		/// </summary>
 		public virtual bool IsHttpHandler (string verb, string uri)
 		{
 			string cacheKey = verb + "_" + uri;
@@ -202,61 +149,35 @@ namespace Mono.WebServer
 
 			locked = false;
 			try {
-#if NET_2_0
 				handlersCacheLock.EnterReadLock ();
-#else
-				handlersCacheLock.AcquireReaderLock (-1);
-#endif
+
 				locked = true;
 				if (handlersCache != null) {
 					bool found;
-#if NET_2_0
 					if (handlersCache.TryGetValue (cacheKey, out found))
 						return found;
-#else
-					if (handlersCache.ContainsKey (cacheKey))
-						return (bool) handlersCache [cacheKey];
-#endif
 				} else {
-			
-#if NET_2_0
 					handlersCache = new Dictionary <string, bool> ();
-#else
-					handlersCache = new Hashtable ();
-#endif
 				}
 			} finally {
-				if (locked) {
-#if NET_2_0
+				if (locked)
 					handlersCacheLock.ExitReadLock ();
-#else
-					handlersCacheLock.ReleaseReaderLock ();
-#endif
-				}
 			}
 			
 			
 			bool handlerFound = LocateHandler (verb, uri);
 			locked = false;
 			try {
-#if NET_2_0
 				handlersCacheLock.EnterWriteLock ();
-#else
-				handlersCacheLock.AcquireWriterLock (-1);
-#endif
+
 				locked = true;
 				if (handlersCache.ContainsKey (cacheKey))
 					handlersCache [cacheKey] = handlerFound;
 				else
 					handlersCache.Add (cacheKey, handlerFound);
 			} finally {
-				if (locked) {
-#if NET_2_0
+				if (locked)
 					handlersCacheLock.ExitWriteLock ();
-#else
-					handlersCacheLock.ReleaseWriterLock ();
-#endif
-				}
 			}
 			
 			return handlerFound;
@@ -264,7 +185,6 @@ namespace Mono.WebServer
 
 		bool LocateHandler (string verb, string uri)
 		{
-#if NET_2_0
 			HttpHandlersSection config = WebConfigurationManager.GetSection ("system.web/httpHandlers") as HttpHandlersSection;
 			HttpHandlerActionCollection handlers = config != null ? config.Handlers : null;
 			int count = handlers != null ? handlers.Count : 0;
@@ -291,11 +211,10 @@ namespace Mono.WebServer
 						return true;
 				}
 			}
-#endif
+
 			return false;
 		}
 
-#if NET_2_0
 		bool PathMatches (HttpHandlerAction handler, string uri)
 		{
 			bool result = false;
@@ -312,7 +231,6 @@ namespace Mono.WebServer
 				
 				string matchExact = null;
 				string endsWith = null;
-				Regex regEx = null;
 
 				if (handlerPath.Length > 0) {
 					if (handlerPath [0] == '*' && (handlerPath.IndexOf ('*', 1) == -1))
@@ -320,8 +238,6 @@ namespace Mono.WebServer
 
 					if (handlerPath.IndexOf ('*') == -1)
 						if (handlerPath [0] != '/') {
-							HttpContext ctx = HttpContext.Current;
-							HttpRequest req = ctx != null ? ctx.Request : null;
 							string vpath = HttpRuntime.AppDomainAppVirtualPath;
 
 							if (vpath == "/")
@@ -364,12 +280,6 @@ namespace Mono.WebServer
 			
 			return result;
 		}
-#else
-		bool PathMatches (object handler, string uri)
-		{
-			return false;
-		}
-#endif
 		
 		string[] SplitVerbs (string verb)
 		{
