@@ -40,10 +40,7 @@ using System.Threading;
 using System.IO;
 using System.Globalization;
 using System.Runtime.InteropServices;
-
-#if NET_2_0
 using System.Collections.Generic;
-#endif
 
 namespace Mono.WebServer
 {
@@ -53,7 +50,6 @@ namespace Mono.WebServer
 		{
 			// There's a hidden missing feature here... :)
 			realUri = uri; pathInfo = String.Empty;
-			string basepath = HttpRuntime.AppDomainAppPath;
 			string vpath = HttpRuntime.AppDomainAppVirtualPath;
 			int vpathLen = vpath.Length;
 			
@@ -69,9 +65,6 @@ namespace Mono.WebServer
 
 			int dot, slash;
 			int lastSlash = uri.Length;
-#if !NET_2_0
-			bool windows = (Path.DirectorySeparatorChar == '\\');
-#endif
 			string partial;
 				
 			for (dot = uri.LastIndexOf ('.'); dot > 0; dot = uri.LastIndexOf ('.', dot - 1)) {
@@ -83,17 +76,8 @@ namespace Mono.WebServer
 				partial = uri.Substring (0, slash);
 				lastSlash = slash;
 
-#if NET_2_0
 				if (!VirtualPathExists (appHost, verb, partial))
 					continue;
-#else
-				if (windows)
-					partial = partial.Replace ('/', '\\');
-				
-				string path = Path.Combine (basepath, (partial));
-				if (!File.Exists (path) && !VirtualPathExists (appHost, verb, partial))
-					continue;
-#endif
 				
 				realUri = vpath + uri.Substring (0, slash);
 				pathInfo = uri.Substring (slash);
@@ -106,11 +90,10 @@ namespace Mono.WebServer
 			if (appHost.IsHttpHandler (verb, uri))
 				return true;
 
-#if NET_2_0
 			VirtualPathProvider vpp = HostingEnvironment.VirtualPathProvider;
 			if (vpp != null && vpp.FileExists ("/" + uri))
 				return true;
-#endif
+
 			return false;
 		}
 	}
